      MODULE NLFPMODULE
c
c     ********************************************************************
c     DEFINE DATA NEEDED FOR NON LINEAR FLOW PROCESS NLFP
c     version 1.0 September 2013
c     
c     ********************************************************************
      IMPLICIT NONE
c
ciril**only need to modify the conductances for Forchheimer         
ciril** create all the variables needed for Forchheimer
ciril-----------------------------------------------------------------------
      REAL CondeffnewR              
      REAL CondeffnewC 
      REAL Klam 
      REAL Criteria 
      REAL, SAVE, POINTER, DIMENSION(:,:,:) :: CondeffoldR
      REAL, SAVE, POINTER, DIMENSION(:,:,:) :: CondeffoldC
      REAL, SAVE, POINTER, DIMENSION(:,:,:) :: Crr
      REAL, SAVE, POINTER, DIMENSION(:,:,:) :: Ccr
      REAL, SAVE, POINTER, DIMENSION(:,:,:) :: bthroughaForch     
ciril** that's all for the moment! ;)
      END MODULE NLFPMODULE
c======================================================================= 
c======================================================================= 
c======================================================================= 
c======================================================================= 
      SUBROUTINE GWF2NLFP1AR(Inunit)
c     ********************************************************************
c     ALLOCATE AND READ DATA FOR FOR NON LINEAR FLOW PROCESS NLFP
c     version 1.0 August 2013 
c
c     ********************************************************************
c
      USE GLOBAL, ONLY:Iout, Ncol, Nrow, Nlay
      USE NLFPMODULE
      IMPLICIT NONE
ciril**need these functions
      INTEGER, INTENT(IN) :: Inunit
ciril** Local Variables
      INTEGER i, K, KK,ii
      CHARACTER(LEN=80) cdummy
      !REAL criteria
      CHARACTER*40 ANAME(1)
      DATA ANAME(1) /'BthroughA Forchheimer'/
c            
ciril**WRITE MESSAGE TO OUTPUT IOUT
ciril**This prove that NLFP is active
      WRITE (Iout, *)
      WRITE (Iout, *) 'NLFP1AR READING FROM UNIT', Inunit
      WRITE (Iout, *) '---------------------------------'
      WRITE (Iout, *) 'NLFP IS ACTIVE (LAYERS) '
      WRITE (Iout, *) '---------------------------------'
ciril**Allocate all the needed variables
ciril------------------------------------------------------------------------
      READ (Inunit, *) cdummy
ciril** read the forcheimer convergence criteria
      READ (Inunit, *) Criteria
      WRITE (Iout, *) 'Forchheimer convergence criteria', Criteria
      ALLOCATE  (bthroughaForch(Ncol,Nrow,Nlay)) 
      ALLOCATE  (CondeffoldR(Ncol,Nrow,Nlay))
      ALLOCATE  (CondeffoldC(Ncol,Nrow,Nlay))
      ALLOCATE  (Crr(Ncol,Nrow,Nlay))
      ALLOCATE  (Ccr(Ncol,Nrow,Nlay))
c
ciril** Call the subroutine U2DREL to read b/a as an array
      DO I=1, Nlay
        K=I
        KK=K
        CALL U2drel(bthroughaForch(:,:,K), ANAME(1), Nrow,Ncol,KK,      &
     &  inunit, iout)
      ENDDO
c
 9002 FORMAT (4X, ' NUMBER OF LAYERS IS', I5)
 9003 FORMAT (4X, I6, 1X, 3I6, 1X, 6I6, 1X, 6I6)
c 
      END SUBROUTINE GWF2NLFP1AR
c======================================================================= 
c======================================================================= 
c======================================================================= 
c======================================================================= 
      SUBROUTINE GWF2NLFPFM (kkiter, kkstp, kkper,ICNVF)
c     ********************************************************************
c     MODIFY THE MODFLOW CONDUCTANCES TO HAVE NON LINEAR FLOW
c     version 1.0 September 2013 
c
c     ********************************************************************
c
      USE GLOBAL, ONLY:Iout, Ncol, Nrow, Nlay, Hold, Hnew, Cr, Cc, Botm,&
     &    Delr, Delc, Ibound,LBOTM
      USE NLFPMODULE
      USE GWFBCFMODULE, ONLY: Hy, LAYCON
c      
c     *************************************************************************
      IMPLICIT NONE
      INTRINSIC ABS 
      INTRINSIC MAX                                                  
! Arguments
      INTEGER, INTENT(IN) :: kkiter, kkstp, kkper
! Local Variables needed 
      REAL DeltaH 
      REAL DeltaX      
      REAL DeltaY                                         
      REAL CondlinR
      REAL CondlinC                                     
      INTEGER i, j, ic, ICNVF
      DOUBLE PRECISION Tick
      DOUBLE PRECISION area
!------------------------------------------------------------------ 
ciril** First initial condition, at the beginning no Forch conductance
       IF (kkiter.EQ.1.and.kkstp.EQ.1.and.kkper.EQ.1)THEN
            Crr=Cr
            Ccr=Cc 
            CondeffoldR=Cr
            CondeffoldC=Cc
       ENDIF
       ICNVF=1
       DO ic = 1, Nlay 
         DO i = 1, Nrow
           DO j = 1, Ncol          
c======================================================================= 
ciril**Flow to right*****************************************************
c======================================================================= 
ciril**don't do this for last column but take the linear conductance  
             IF (j.EQ.Ncol) THEN
                 CondeffnewR=0
ciril** take also the linear conductance if the cell as an inactive cell nearby 
             ELSEIF (j.LT.Ncol.AND.Ibound(j+1,i,ic).EQ.0)THEN
                 CondeffnewR=Cr(j,i,ic)  
ciril** else begin the computation of DeltaX and Klam and the linear conductance
             ELSE
                 DeltaX=(Delr(j)+Delr(j+1))/2 
ciril** computation of DeltaH    
                 DeltaH = (Hnew(j,i,ic)-Hnew(j+1,i,ic))
                 DeltaH=ABS(DeltaH)   
ciril** computation of Klam
ciril** test for laycon=> unconfined case (Layer type 1 or 3)
                 IF (LAYCON(ic).EQ.1.OR.LAYCON(ic).EQ.3) THEN
                      Klam=Hy(j,i,ic)
                      CondlinR =Cr(j,i,ic)
                      !CondlinR =Crr(j,i,ic)                      
ciril** otherwise confined case (layer type 0 or 2 and T fixed constant)
                 ELSE
                      tick=Botm(j,i,(ic)-1)-Botm(j,i,(ic))
                      area=Delc(i)*tick
                      Klam=(Crr(j,i,(ic))*Deltax)/area
                      CondlinR=Crr(j,i,ic) 
                 ENDIF
ciril** test for dry cells to avoid an error message
                 IF (CondlinR.EQ.0) THEN
                      CondeffnewR=0
                 ELSE                   
ciril** computation of the Forchheimer conductance
             CondeffnewR = (CondlinR)/(1+                               &
     & ((CondeffoldR(j,i,ic)/CondlinR)*                                 &
     & ((Klam)/(DeltaX))*(bthroughaForch(j,i,ic))*(DeltaH))) 
                 ENDIF 
             ENDIF
ciril** Forchheimer convergence criteria 
            IF (ABS(CondeffoldR(j,i,ic)-CondeffnewR)/                   &
     &       MAX(CondeffoldR(j,i,ic),CondeffnewR).GT.Criteria)THEN
             ICNVF=0
            ENDIF
ciril** the Forchheimer conductance is save in a matrix and in MODFLOW                                         
             CondeffoldR(j,i,ic)=CondeffnewR
             Cr(j,i,ic)=CondeffoldR(j,i,ic)                       
ciril**enddo for j loop and then for i loop
           ENDDO
         ENDDO
c======================================================================= 
ciril**Flow to front****************************************************
c======================================================================= 
         DO i = 1, Nrow
           DO j = 1, Ncol
ciril**don't do this for last column but take the linear conductance        
             IF (i.EQ.Nrow) THEN
                 CondeffnewC=Cc(j,i,ic)
ciril** take also the linear conductance if the cell as an inactive cell nearby
             ELSEIF (i.LT.Nrow.AND.Ibound(j,i+1,ic).EQ.0)THEN
                 CondeffnewC=Cc(j,i,ic)                
             ELSE
                 DeltaY=(Delc(i)+Delc(i+1))/2 
ciril** computation of DeltaH
                 DeltaH=(Hnew(j,i,ic)-Hnew(j,i+1,ic))
                 DeltaH=ABS(DeltaH) 
ciril** computation of Klam
ciril** test for laycon=> unconfined case (Layer type 1 or 3)
                 IF (LAYCON(ic).EQ.1.OR.LAYCON(ic).EQ.3) THEN
                      Klam=Hy(j,i,ic)
                      CondlinC=Cc(j,i,ic)
                      !CondlinC=Ccr(j,i,ic)
                 ELSE
                      tick=Botm(j,i,(ic)-1)-Botm(j,i,(ic))
                      area=(Delr(j)*tick)
                      Klam=(Ccr(j,i,ic)*DeltaY)/area 
                      CondlinC=Ccr(j,i,ic) 
                 ENDIF
ciril** test for dry cells to avoid an error message
                 IF (CondlinC.EQ.0) THEN
                      CondeffnewC=0
                 ELSE
ciril** computation of the Forchheimer conductance
        CondeffnewC = (CondlinC)/(1+                                    &
     & ((CondeffoldC(j,i,ic)/CondlinC)*                                 &
     & ((Klam)/(DeltaY))*(bthroughaForch(j,i,ic))*(DeltaH)))
                 ENDIF
             ENDIF
ciril** Forchheimer convergence criteria 
             IF (ABS(CondeffoldC(j,i,ic)-CondeffnewC)/                  &
     &        MAX(CondeffoldC(j,i,ic),CondeffnewC).GT.Criteria)THEN
              ICNVF=0
             ENDIF 
ciril** the Forchheimer conductance is save in a matrix and in MODFLOW    
             CondeffoldC(j,i,ic)=CondeffnewC 
             Cc(j,i,ic)=CondeffoldC(j,i,ic)
ciril** open and write the results in a file in order to cheeck it later
!      OPEN (Unit=999, FILE='Condeff_output.txt',status="unknown",       &
!     & action="write")
!      WRITE (999,'(a15,i4,x,a15,i4,x,a15,i4,1x,a15,i4,1x,               &
!     & a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,&
!     & a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,a15,e11.5,1x,&
!     &  a15,e11.5,1x,a15,e11.5,1x)')                                    &
!     &'row=',i,'column=',j,                                             &
!     &'kkstp=',kkstp, 'kkiter=',kkiter, 'CondeffoldR=',                 &
!     & CondeffoldR(j,i,ic), 'CondeffoldC=',CondeffoldC(j,i,ic),'Hold=', &
!     & Hold(j,i,ic), 'Hnew=',Hnew(j,i,ic), 'Crr=',Crr(j,i,ic), 'Ccr=',  &
!     & Ccr(j,i,ic),'CondlinR=',CondlinR        ,'CondeffnewR=',         &
!     & CondeffnewR,'CondeffnewC=',CondeffnewC,'Cr=',Cr(j,i,ic),         &
!     &'Cc=',Cc(j,i,ic)
ciril**enddo for j loop, for i loop and finally for ic loop
           ENDDO
         ENDDO
       ENDDO
      END SUBROUTINE GWF2NLFPFM
c======================================================================= 
c======================================================================= 
c======================================================================= 
c======================================================================= 
